C>>
C>> This program is to generate the POWER distribution file
C   for running DANTE induction heating models
C>>------------------------------------------------------------------
         PROGRAM IH_POWER_CREATE
C
       IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)

       CHARACTER FILNAM1*70, OFILE*70
       CHARACTER*255 TEXTA
       CHARACTER INSTNAM*70
C
       CHARACTER STRNG1*225, STRNG2*225
C
       DIMENSION FLUXA(2), TIMEA(2), COORDSA(3)
       CHARACTER*80 SNAME
C
C>> IH PROCESS Information
C..  TRAVEL_SPEED: Inductor Travel Speed
C..  WIDTH_INDUCTOR: Width of Inudcot
C..  P0_INDUCTOR: Inductor initial position
C..  NUM_FRAME_TOTAL: Total Number of Frames
C..  TIME_STEP: Time increment per frame
C..  POWER_WGT: Weight factor of Power
       COMMON /IH_PROCESS/ TRAVEL_SPEED,WIDTH_INDUCTOR,P0_INDUCTOR,
     &         TIME_STEP,POWER_WGT,NUM_FRAME_TOTAL
C
C>>----------------------------------------------
C>> Information need to be initialzed
C>> Iductor Travel Speed
       TRAVEL_SPEED=10.D0
C>> Width of Inductor
       WIDTH_INDUCTOR=12.D0
C>> Initial position of Inudctor
C..       P0_INDUCTOR=-25.D0
       P0_INDUCTOR=-12.D0
C>> Total Frame number
       NUM_FRAME_TOTAL=77
C>> Frame time increment
       TIME_STEP=0.15D0
C>> Power Weight
       POWER_WGT=  1050.0/280.0*2.0 !1.D0
C
C>> Write info for the file
       TIME_INITIAL=0.d0
C>> Starting Frame number: needs to be 1 
       NUM_FRAME_CURRENT=1
C
C>> Read in the file 
       WRITE(*,*) 'Input Element-Integration-Coordinate File Name:'
       WRITE(*,*)
       READ(*,1000) FILNAM1 
C
C>> Output
C>> Open the iutput file name:
       NLEN=LEN_TRIM(FILNAM1)
       NFILE=0
       DO I=1,NLEN
         IF (FILNAM1(I:I).NE.'.' .AND. FILNAM1(I:I).NE.' ') THEN 
           NFILE=NFILE+1
         ELSE
           GOTO 120
         ENDIF
       END DO
120    CONTINUE
C
C>>
       OFILE=FILNAM1(1:NFILE)//'_POWER.OUT'
C> Open the output file for writing
       OPEN(16,FILE=OFILE, STATUS='UNKNOWN')
       WRITE(16,1000) '** Power File for the Blade Model'
       WRITE(16,1010)  FILNAM1(1:NLEN)
       WRITE(16,1000) '** '
       WRITE(16,1000) '*POWER_CHECK'
       WRITE(16,1000) ' 0'
       WRITE(16,1000) '*NUM_POWER_FRAME'
       WRITE(16,*)    NUM_FRAME_TOTAL
       WRITE(16,1000) '*POWER_WGT'
       WRITE(16,*)    POWER_WGT
       WRITE(16,1000) '**' 
C
C>> Open the Element-coordinate file
       OPEN(UNIT=12, FILE=FILNAM1, STATUS='OLD', ERR=160)
       GOTO 180
160    CONTINUE
       WRITE(*,*) 'Can not open Coordinate File, program terminated!'
       WRITE(*,*)  FILNAM1(1:NLEN)
       STOP
180    CONTINUE
C
C>>--------------------------------------------------------
C
150    CONTINUE
       POWER_TIME=TIME_INITIAL+(NUM_FRAME_CURRENT-1)*TIME_STEP
C
       WRITE(*,*) 'FRAME NUMBER: ', NUM_FRAME_CURRENT 
       WRITE(*,*) '  FRAME TIME: ', POWER_TIME 
C
       TIMEA(1)=POWER_TIME
       TIMEA(2)=TIMEA(1)
C
       WRITE(16,1000) '*START_POWER'
       WRITE(16,1000) '*POWER_TIME'
       WRITE(16,*)     POWER_TIME
       WRITE(16,1000) '*POWER_DENSITY'
C
C>> Processing the Input File
200    CONTINUE
       READ(12, 1000, ERR=3333) TEXTA
       GOTO 3334
3333   WRITE(7,*) 'Error after reading line: ', TEXTA
       WRITE(7,*) 'The line will be ignored!'
C>>       STOP
       GOTO 3500
3334   CONTINUE
C>> Change everything to upper case
       CALL TO_UCASE2(TEXTA)
       CALL ANAVAL(TEXTA, X0,Y0,Z0, IDATA,ID_ELE,ID_ITG)
C
      IF(IDATA.EQ.0) THEN  !not a data line, do nothing
C>>      
C..         NLEN1=LEN_TRIM(TEXTA)
C..         STRNG1='** '//TEXTA(1:NLEN1)
C..         WRITE(16,1000) STRNG1
C
       ELSE IF(IDATA.EQ.1) THEN  !is data line
C
         FLUXA(1)=0.D0
         FLUXA(2)=0.D0
C
         COORDSA(1)=X0
         COORDSA(2)=Y0
         COORDSA(3)=Z0
C
         KSTEP=1
         KINC=1
         NOEL=ID_ELE
         NPT=ID_ITG
C
         CALL DFLUX(FLUXA,SOL,KSTEP,KINC,TIMEA,NOEL,NPT,
     &     COORDSA,JLTYP,TEMP,PRESS,SNAME)
C
         WRITE(STRNG1, 1500) ID_ELE, ID_ITG, FLUXA(1)
1500     FORMAT(I12,','2X,I4,',',2X, E15.6)
C
C>> String Process
         CALL STRNG_PROCESS(STRNG1,STRNG2,NSTRNG)
         WRITE(16,1000) STRNG2(1:NSTRNG)
C
       ELSE
         WRITE(7,*) 'Unexpected Error, Stop!'
         STOP
       ENDIF
C>> Read till the end of the file
3500   CONTINUE
       IF(.NOT. EOF(12)) GOTO 200
C
       NUM_FRAME_CURRENT=NUM_FRAME_CURRENT+1
       IF(NUM_FRAME_CURRENT.LE.NUM_FRAME_TOTAL) THEN
         REWIND(12)
         GOTO 150
       ELSE
         WRITE(16,1000) '*END_POWER'
       ENDIF
C
C>> Close the input file
       CLOSE(12)
C>>
C
C>> CLOSE Output File
       WRITE(16,1000) '**End-Of-File'
       CLOSE(16)
       WRITE(*,*) 'Program done'
C>>
       STOP
C>>-----------------------------------------------------------------
1000   FORMAT(A)
1010   FORMAT('** Input Carbon File name:  ', A)
C>>
       END

C>>------------------------------------------------------------------
C>> Dlfux single shot GM crankshaft, R=34.45mm, Frequency=12KHZ
       SUBROUTINE DFLUX(FLUX,SOL,KSTEP,KINC,TIME,NOEL,NPT,
     &     COORDS,JLTYP,TEMP,PRESS,SNAME)

       IMPLICIT REAL*8(A-H,O-Z), INTEGER*4(I-N)
C
       DIMENSION FLUX(2), TIME(2),COORDS(3)
       CHARACTER*80 SNAME
C
C>> IH PROCESS Information
C..  TRAVEL_SPEED: Inductor Travel Speed
C..  WIDTH_INDUCTOR: Width of Inudcot
C..  P0_INDUCTOR: Inductor initial position
C..  NUM_FRAME_TOTAL: Total Number of Frames
C..  TIME_STEP: Time increment per frame
C..  POWER_WGT: Weight factor of Power
       COMMON /IH_PROCESS/ TRAVEL_SPEED,WIDTH_INDUCTOR,P0_INDUCTOR,
     &         TIME_STEP,POWER_WGT,NUM_FRAME_TOTAL
C
C>>-----------------------------
       DIMENSION HGEN(10)
       DIMENSION HDEPTH(10)
C>>-----------------------------
       DIMENSION HGENA(10)
       DIMENSION HDEPTHA(10)
C
C>> For Recipe A
       DATA (HDEPTHA(I),I=1,10) /1.D0, 2.D0, 3.D0, 4.D0, 5.D0,
     &                          6.D0, 7.D0, 8.D0, 9.D0, 10.D0 /
       DATA (HGENA(I),I=1,10) /1.25D0,0.90D0,0.61D0,0.38D0,0.17D0,
     &                        0.05D0,0.00D0,0.00D0,0.00D0,0.00D0 /
C
C>..       DATA (HDEPTHA(I),I=1,10) /0.D0, 0.1D0, 0.2D0, 0.3D0, 0.4D0,
C     &                          0.5D0, 0.6D0, 0.7D0, 0.8D0, 0.9D0 /
C       DATA (HGENA(I),I=1,10) /2.00D1,1.00D1,0.50D0,0.00D0,0.00D0,
C     &                        0.00D0,0.00D0,0.00D0,0.00D0,0.00D0 /
C>>-----------------------------
C>>--------------------------------------------------------------------
       FLUX(1) = 0.D0
       FLUX(2) = 0.D0
C
C>> The induction heating is only used in the first step for this set up.
C++        IF (KSTEP .EQ. 1) THEN
       WEIGHT=1.0D0  !Recipe A
       DO I=1, 10
         HGEN(I) = HGENA(I)
         HDEPTH(I) = HDEPTHA(I)
       END DO
C
C>> Travel Speed
C..       TRAVEL_SPEED=5.D0
C>> Inductor Position
C..       WIDTH_INDUCTOR=25.D0
C>> Inductor initial start position
C..       P0_INDUCTOR=-25.D0 !Z direction
C
C>> Current Step Time
       TIME1=TIME(1)
C
C>> Current position of inductor
       STRAT_INUDCTOR=P0_INDUCTOR+TIME1*TRAVEL_SPEED
       END_INUDCTOR=STRAT_INUDCTOR+WIDTH_INDUCTOR
C
C>> Find the heat flux for the specific integration point
       X0=COORDS(1)
       Y0=COORDS(2)
       Z0=COORDS(3)
C>>
       IF((Y0.LT.STRAT_INUDCTOR).OR.(Y0.GT.END_INUDCTOR)) THEN
         ! Outside the ineductor
         RETURN
       ENDIF
C
C>>-------------------------------------------------------------------
C>>-------------------------------------------------------------------
C>> Top Arc Angle
C>> Note: Center (0,0) first
         AA=X0-0.D0
         RR_DIST = X0
         RR_PART = 12.7D0
         DEPTH=RR_PART - RR_DIST
         WEIGHT=1.0D0  !100% power
C
       IF(DEPTH.GT.1.D0) RETURN
C>> Points for interpolation
       JP1=-1
C       IF(DEPTH.GE.0.D0 .AND. DEPTH.LT.HDEPTH(1)) THEN
       IF(DEPTH.LT.HDEPTH(1)) THEN
         JP1 = 1
       ELSE IF(DEPTH.GE.HDEPTH(1) .AND. DEPTH.LT.HDEPTH(2)) THEN
         JP1 = 2
       ELSE IF(DEPTH.GE.HDEPTH(2) .AND. DEPTH.LT.HDEPTH(3)) THEN
         JP1 = 3
       ELSE IF(DEPTH.GE.HDEPTH(3) .AND. DEPTH.LT.HDEPTH(4)) THEN
         JP1 = 4
       ELSE IF(DEPTH.GE.HDEPTH(4) .AND. DEPTH.LT.HDEPTH(5)) THEN
         JP1 = 5
       ELSE IF(DEPTH.GE.HDEPTH(5) .AND. DEPTH.LT.HDEPTH(6)) THEN
         JP1 = 6
       ELSE IF(DEPTH.GE.HDEPTH(6) .AND. DEPTH.LT.HDEPTH(7)) THEN
         JP1 = 7
       ELSE IF(DEPTH.GE.HDEPTH(7) .AND. DEPTH.LT.HDEPTH(8)) THEN
         JP1 = 8
       ELSE IF(DEPTH.GE.HDEPTH(8) .AND. DEPTH.LT.HDEPTH(9)) THEN
         JP1 = 9
       ELSE IF(DEPTH.GE.HDEPTH(9) .AND. DEPTH.LE.HDEPTH(10)) THEN
         JP1 = 10
       ENDIF
C
C>> Interpolation
       IF(JP1.LE.1) THEN
         FLUX(1) = HGEN(1)
       ELSE IF(JP1.GT.10) THEN
         FLUX(1) = 0.D0
       ELSE !JP1 >=2, <=10, Interpolation
         FLUX(1)=HGEN(JP1-1)+(HGEN(JP1)-HGEN(JP1-1))*
     &           (DEPTH-HDEPTH(JP1-1))/(HDEPTH(JP1)-HDEPTH(JP1-1))
       ENDIF
C>> Adding Weight
       FLUX(1)=FLUX(1)*WEIGHT
C
C>>
       RETURN
       END
C
C>>********************************************************************
         SUBROUTINE TO_UCASE2(WORD)
       IMPLICIT REAL*8(A-H,O-Z), INTEGER*4(I-N)
C>> Convert lower case string to upper case string
       CHARACTER*255 WORD

       NLEN = LEN_TRIM(WORD)
       DO I=1,NLEN
         IC = ICHAR(WORD(I:I))
         IF(IC.GE.97 .AND. IC.LE.122) WORD(I:I)=CHAR(IC-32)
       END DO

       RETURN
       END

C>>********************************************************************
         SUBROUTINE ANAVAL(TXTS,X0,Y0,Z0, IDATA,ID_ELE,ID_ITG)
       IMPLICIT REAL*8(A-H,O-Z), INTEGER*4(I-N)

       CHARACTER*255 TXTS
C>>------------------------------------------------------------------
       IDATA=0  !not a data line

       NLEN1=LEN_TRIM(TXTS)
       IF(NLEN1.EQ.0) RETURN
       
       READ(TXTS, *,ERR=3333) ID_ELE, ID_ITG, X0,Y0,Z0
       GOTO 3334
3333   CONTINUE
       ID_ELE = 0
       ID_ITG = 0
       X0=0.D0
       Y0=0.D0
       Z0=0.D0
C
       RETURN
       
3334   CONTINUE
       IF(ID_ELE.GT.0) IDATA=1  !is data line

       RETURN
       END
C
C>>********************************************************************
       SUBROUTINE STRNG_PROCESS(STRG1, STRG2, NLENG)
       IMPLICIT REAL*8(A-H,O-Z), INTEGER*4(I-N)

       CHARACTER STRG1*225, STRG2*225

       LTOT=LEN_TRIM(STRG1)

       NLENG=0 
       DO I=1, LTOT
         IF(STRG1(I:I) .NE. ' ') THEN
           IF(STRG1(I:I) .EQ. ',') THEN
             NLENG=NLENG+1 
             STRG2(NLENG:NLENG)=STRG1(I:I)
C
             STRG2(NLENG+1:NLENG+4)='    '
             NLENG=NLENG+4 
           ELSE
             NLENG=NLENG+1 
             STRG2(NLENG:NLENG)=STRG1(I:I)
           ENDIF
         ENDIF
       END DO

       RETURN
       END
C>>********************************************************************

